/*
 *  powerpipo          business presentation software
 *  Copyright (c) 2007 Sam Hocevar <sam@zoy.org>
 *                All Rights Reserved
 *
 *  $Id$
 *
 *  This program is free software. It comes without any warranty, to
 *  the extent permitted by applicable law. You can redistribute it
 *  and/or modify it under the terms of the Do What The Fuck You Want
 *  To Public License, Version 2, as published by Sam Hocevar. See
 *  http://sam.zoy.org/wtfpl/COPYING for more details.
 */

#include "config.h"

#include <stdio.h>

#include <caca.h>
#include <caca.h>

static void do_page(caca_canvas_t *, int);
static void runtoilet(caca_canvas_t *, char const *, char const *);

int main(int argc, char *argv[])
{
    caca_display_t *dp;
    caca_canvas_t *cv;
    int page = 0, npages = 10, changed = 1;

    cv = caca_create_canvas(0, 0);

    dp = caca_create_display(cv);
    if(!dp)
        return 1;

    caca_set_display_time(dp, 20000);
    caca_set_display_title(dp, "PowerPIPO presentation tool");

    for(;;)
    {
        caca_event_t ev;

        /* Handle events */
        while(caca_get_event(dp, CACA_EVENT_KEY_PRESS
                                  | CACA_EVENT_QUIT, &ev, 10000))
        {
            if(ev.type == CACA_EVENT_QUIT)
                goto end;
            switch(ev.data.key.ch)
            {
            case CACA_KEY_ESCAPE:
                goto end;
                break;
            case ' ':
            case CACA_KEY_PAGEDOWN:
                page++;
                if(page > npages - 1)
                    page = npages - 1;
                changed = 1;
                break;
            case CACA_KEY_PAGEUP:
                page--;
                if(page < 0)
                    page = 0;
                changed = 1;
                break;
            }
        }

        if(changed)
        {
            do_page(cv, page);
            caca_refresh_display(dp);
            changed = 0;
        }
    }

 end:
    caca_free_display(dp);
    caca_free_canvas(cv);

    return 0;
}

static void do_page(caca_canvas_t *cv, int page)
{
    char str[BUFSIZ];
    caca_canvas_t *tmp;

    caca_clear_canvas(cv);
    tmp = caca_create_canvas(0, 0);

    if(page < 1) goto end;
    sprintf(str, "\x1b[33;93m» \x1b[37;97mEfficient presentations");
    runtoilet(tmp, "-f smmono12", str);
    caca_blit(cv, 4, 0, tmp, NULL);

    if(page < 2) goto end;
    sprintf(str, "\x1b[32;92m» \x1b[37;97mWhy text mode?");
    runtoilet(tmp, "-f smmono9", str);
    caca_blit(cv, 11, 9, tmp, NULL);

    if(page < 3) goto end;
    sprintf(str, "\x1b[32;92m» \x1b[37;97mExample: \x1b[34mp\x1b[35mo\x1b[36mw\x1b[32me\x1b[31mr\x1b[34mp\x1b[35mi\x1b[36mp\x1b[32mo\x1b[37m™");
    runtoilet(tmp, "-f smmono9", str);
    caca_blit(cv, 11, 16, tmp, NULL);

    if(page < 4) goto end;
    sprintf(str, "\x1b[33;93m» \x1b[37;97mAdvanced stuff");
    runtoilet(tmp, "-f smmono12", str);
    caca_blit(cv, 4, 24, tmp, NULL);

    if(page < 5) goto end;
    sprintf(str, "\x1b[32;92m» \x1b[37;97mTransition effects");
    runtoilet(tmp, "-f smmono9", str);
    caca_blit(cv, 11, 34, tmp, NULL);

    if(page < 6) goto end;
    sprintf(str, "\x1b[32;92m» \x1b[37;97mColor filters");
    runtoilet(tmp, "-f smmono9 --gay", str);
    caca_blit(cv, 11, 41, tmp, NULL);

    if(page < 7) goto end;
    sprintf(str, "\x1b[32;92m» \x1b[37;97mPictures and movies");
    runtoilet(tmp, "-f smmono9", str);
    caca_blit(cv, 11, 48, tmp, NULL);

    if(page < 8) goto end;
    sprintf(str, "\x1b[32;92m» \x1b[37;97mEmbedded applications");
    runtoilet(tmp, "-f smmono9", str);
    caca_blit(cv, 11, 55, tmp, NULL);

end:
    caca_printf(cv, 0, 0, "page %i", page);
}

/* Warning: quotes are not safe */
static void runtoilet(caca_canvas_t *cv, char const *options, char const *str)
{
    char buf[BUFSIZ], cmd[BUFSIZ];
    FILE *f;
    int ret;

    caca_set_canvas_size(cv, 0, 0);

    sprintf(cmd, "toilet -w 1000 %s '%s'", options, str);
    f = popen(cmd, "r");
    if(!f)
        return;
    ret = fread(buf, 1, BUFSIZ, f);
    fclose(f);
    if(ret <= 0)
        return;

    ret = caca_import_memory(cv, buf, ret, "utf8");
    if(ret <= 0)
        return;
}

